'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
var crypto = require('crypto');
var _ = require('lodash');
var Draggable = require('vuedraggable');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        for (var k in e) {
            n[k] = e[k];
        }
    }
    n["default"] = e;
    return Object.freeze(n);
}

var crypto__default = /*#__PURE__*/_interopDefaultLegacy(crypto);
var ___namespace = /*#__PURE__*/_interopNamespace(_);
var Draggable__default = /*#__PURE__*/_interopDefaultLegacy(Draggable);

const clone = value => {
    return JSON.parse(JSON.stringify(value));
};
const useOriginal = value => {
    const original = vue.reactive({
        raw: clone(value),
        stringified: JSON.stringify(value),
        update(value) {
            this.raw = clone(value);
            this.stringified = JSON.stringify(value);
        },
        matches(value) {
            return JSON.stringify(value) == this.stringified;
        },
    });
    return original;
};

const useIndex = function useIndex({ load, filters = {}, sortBy = [], }) {
    const search = vue.ref('');
    let index = vue.reactive({
        isLoading: false,
        items: [],
        meta: {},
        search,
        filters,
        sortBy,
        async load() {
            this.isLoading = true;
            return load(this.__getParams())
                .then(response => {
                this.items = response.data.data;
                this.meta = response.data.meta;
                return response;
            })
                .finally(() => (this.isLoading = false));
        },
        reloadOnChange(value) {
            vue.watch(value, () => this.load());
        },
        setFilter(type, value) {
            if (!this.filters[type]) {
                this.filters[type] = {};
            }
            this.filters[type].value = value;
            this.load();
        },
        removeFilter(type) {
            const value = this.filters[type].value;
            if (Array.isArray(value)) {
                this.filters[type] = [];
            }
            else if (typeof value === 'object') {
                this.filters[type] = {};
            }
            else {
                this.filters[type] = undefined;
            }
            this.load();
        },
        setPage(newPage) {
            if (this.newPage < 1 || this.newPage > this.lastPage) {
                return;
            }
            this.currentPage = newPage;
            this.load();
        },
        setNextPage() {
            this.setPage(this.currentPage + 1);
        },
        setPrevPage() {
            this.setPage(this.currentPage - 1);
        },
        setLastPage() {
            this.setPage(this.getLastPage());
        },
        updateSearch(e) {
            search.value =
                e instanceof Event
                    ? e.target.value
                    : e;
            this.setPage(1);
        },
        isSortedBy(key, direction = null) {
            for (let i = 0; i < this.sortBy.length; i++) {
                let ordering = this.sortBy[i];
                if (!ordering || ordering.key != key)
                    continue;
                return direction ? ordering.direction == direction : true;
            }
        },
        addSortBy(key, direction = 'asc') {
            this.removeSortBy(key, false);
            this.sortBy.push({ key, direction });
            this.load();
        },
        removeSortBy(key, reload = true) {
            const sorting = clone(this.sortBy);
            for (let i = 0; i < sorting.length; i++) {
                if (sorting[i] && sorting[i].key == key)
                    delete this.sortBy[i];
            }
            if (reload)
                this.load();
        },
        __getParams() {
            let params = {
                page: this.currentPage,
                search: this.search,
                sortBy: [],
            };
            this.sortBy.forEach(element => {
                params.sortBy.push(`${element.key}.${element.direction}`);
            });
            for (let key in this.filters) {
                if (!this.filters[key]) {
                    continue;
                }
                params[`filter.${key}`] =
                    'transform' in this.filters[key]
                        ? this.filters[key].transform(this.filters[key].value)
                        : this.filters[key].value;
            }
            for (let key in params) {
                if (!params[key])
                    delete params[key];
            }
            return params;
        },
    });
    index.reloadOnChange(search);
    return index;
};

const useForm = function ({ data, submit, load = async (id) => { }, transform = data => data, onDirty = () => { }, }) {
    let form = vue.reactive({
        ...data,
        errors: {},
        isDirty: false,
        isSubmitting: false,
        isLoading: false,
        original: useOriginal(data),
        __id: undefined,
        load(id) {
            this.isLoading = true;
            this.__id = id;
            return load(id)
                .then(response => {
                this.setData(response.data.data);
                this.isLoading = false;
                return response;
            })
                .catch(e => {
                this.isLoading = false;
                throw e;
            });
        },
        data() {
            return Object.keys(data).reduce((carry, key) => {
                carry[key] = this[key];
                return carry;
            }, {});
        },
        setData(newData) {
            Object.keys(data).forEach(key => (this[key] = newData[key]));
            this.original.update(this.data());
            this.isDirty = false;
        },
        submit(e) {
            if (e instanceof Event) {
                e.preventDefault();
            }
            this.isSubmitting = true;
            const data = transform(this.data());
            return submit(data, this.__id)
                .then(response => {
                this.errors = {};
                this.original.update(this.data());
                this.isSubmitting = false;
                return response;
            })
                .catch(e => {
                let data = e.response.data;
                if ('errors' in data) {
                    this.errors = data.errors;
                }
                this.isSubmitting = false;
                throw e;
            });
        },
    });
    onDirty.bind(form);
    vue.watch(form, () => {
        form.isDirty = !form.original.matches(form.data());
        if (form.isDirty) {
            onDirty(form);
        }
    }, { immediate: true, deep: true });
    return form;
};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var rng_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = rng;

var _crypto = _interopRequireDefault(crypto__default);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const rnds8Pool = new Uint8Array(256); // # of random values to pre-allocate

let poolPtr = rnds8Pool.length;

function rng() {
  if (poolPtr > rnds8Pool.length - 16) {
    _crypto.default.randomFillSync(rnds8Pool);

    poolPtr = 0;
  }

  return rnds8Pool.slice(poolPtr, poolPtr += 16);
}
});

unwrapExports(rng_1);

var regex = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _default = /^(?:[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}|00000000-0000-0000-0000-000000000000)$/i;
exports.default = _default;
});

unwrapExports(regex);

var validate_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regex = _interopRequireDefault(regex);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function validate(uuid) {
  return typeof uuid === 'string' && _regex.default.test(uuid);
}

var _default = validate;
exports.default = _default;
});

unwrapExports(validate_1);

var stringify_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _validate = _interopRequireDefault(validate_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Convert array of 16 byte values to UUID string format of the form:
 * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
 */
const byteToHex = [];

for (let i = 0; i < 256; ++i) {
  byteToHex.push((i + 0x100).toString(16).substr(1));
}

function stringify(arr, offset = 0) {
  // Note: Be careful editing this code!  It's been tuned for performance
  // and works in ways you may not expect. See https://github.com/uuidjs/uuid/pull/434
  const uuid = (byteToHex[arr[offset + 0]] + byteToHex[arr[offset + 1]] + byteToHex[arr[offset + 2]] + byteToHex[arr[offset + 3]] + '-' + byteToHex[arr[offset + 4]] + byteToHex[arr[offset + 5]] + '-' + byteToHex[arr[offset + 6]] + byteToHex[arr[offset + 7]] + '-' + byteToHex[arr[offset + 8]] + byteToHex[arr[offset + 9]] + '-' + byteToHex[arr[offset + 10]] + byteToHex[arr[offset + 11]] + byteToHex[arr[offset + 12]] + byteToHex[arr[offset + 13]] + byteToHex[arr[offset + 14]] + byteToHex[arr[offset + 15]]).toLowerCase(); // Consistency check for valid UUID.  If this throws, it's likely due to one
  // of the following:
  // - One or more input array values don't map to a hex octet (leading to
  // "undefined" in the uuid)
  // - Invalid input values for the RFC `version` or `variant` fields

  if (!(0, _validate.default)(uuid)) {
    throw TypeError('Stringified UUID is invalid');
  }

  return uuid;
}

var _default = stringify;
exports.default = _default;
});

unwrapExports(stringify_1);

var v1_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _rng = _interopRequireDefault(rng_1);

var _stringify = _interopRequireDefault(stringify_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// **`v1()` - Generate time-based UUID**
//
// Inspired by https://github.com/LiosK/UUID.js
// and http://docs.python.org/library/uuid.html
let _nodeId;

let _clockseq; // Previous uuid creation time


let _lastMSecs = 0;
let _lastNSecs = 0; // See https://github.com/uuidjs/uuid for API details

function v1(options, buf, offset) {
  let i = buf && offset || 0;
  const b = buf || new Array(16);
  options = options || {};
  let node = options.node || _nodeId;
  let clockseq = options.clockseq !== undefined ? options.clockseq : _clockseq; // node and clockseq need to be initialized to random values if they're not
  // specified.  We do this lazily to minimize issues related to insufficient
  // system entropy.  See #189

  if (node == null || clockseq == null) {
    const seedBytes = options.random || (options.rng || _rng.default)();

    if (node == null) {
      // Per 4.5, create and 48-bit node id, (47 random bits + multicast bit = 1)
      node = _nodeId = [seedBytes[0] | 0x01, seedBytes[1], seedBytes[2], seedBytes[3], seedBytes[4], seedBytes[5]];
    }

    if (clockseq == null) {
      // Per 4.2.2, randomize (14 bit) clockseq
      clockseq = _clockseq = (seedBytes[6] << 8 | seedBytes[7]) & 0x3fff;
    }
  } // UUID timestamps are 100 nano-second units since the Gregorian epoch,
  // (1582-10-15 00:00).  JSNumbers aren't precise enough for this, so
  // time is handled internally as 'msecs' (integer milliseconds) and 'nsecs'
  // (100-nanoseconds offset from msecs) since unix epoch, 1970-01-01 00:00.


  let msecs = options.msecs !== undefined ? options.msecs : Date.now(); // Per 4.2.1.2, use count of uuid's generated during the current clock
  // cycle to simulate higher resolution clock

  let nsecs = options.nsecs !== undefined ? options.nsecs : _lastNSecs + 1; // Time since last uuid creation (in msecs)

  const dt = msecs - _lastMSecs + (nsecs - _lastNSecs) / 10000; // Per 4.2.1.2, Bump clockseq on clock regression

  if (dt < 0 && options.clockseq === undefined) {
    clockseq = clockseq + 1 & 0x3fff;
  } // Reset nsecs if clock regresses (new clockseq) or we've moved onto a new
  // time interval


  if ((dt < 0 || msecs > _lastMSecs) && options.nsecs === undefined) {
    nsecs = 0;
  } // Per 4.2.1.2 Throw error if too many uuids are requested


  if (nsecs >= 10000) {
    throw new Error("uuid.v1(): Can't create more than 10M uuids/sec");
  }

  _lastMSecs = msecs;
  _lastNSecs = nsecs;
  _clockseq = clockseq; // Per 4.1.4 - Convert from unix epoch to Gregorian epoch

  msecs += 12219292800000; // `time_low`

  const tl = ((msecs & 0xfffffff) * 10000 + nsecs) % 0x100000000;
  b[i++] = tl >>> 24 & 0xff;
  b[i++] = tl >>> 16 & 0xff;
  b[i++] = tl >>> 8 & 0xff;
  b[i++] = tl & 0xff; // `time_mid`

  const tmh = msecs / 0x100000000 * 10000 & 0xfffffff;
  b[i++] = tmh >>> 8 & 0xff;
  b[i++] = tmh & 0xff; // `time_high_and_version`

  b[i++] = tmh >>> 24 & 0xf | 0x10; // include version

  b[i++] = tmh >>> 16 & 0xff; // `clock_seq_hi_and_reserved` (Per 4.2.2 - include variant)

  b[i++] = clockseq >>> 8 | 0x80; // `clock_seq_low`

  b[i++] = clockseq & 0xff; // `node`

  for (let n = 0; n < 6; ++n) {
    b[i + n] = node[n];
  }

  return buf || (0, _stringify.default)(b);
}

var _default = v1;
exports.default = _default;
});

unwrapExports(v1_1);

var parse_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _validate = _interopRequireDefault(validate_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parse(uuid) {
  if (!(0, _validate.default)(uuid)) {
    throw TypeError('Invalid UUID');
  }

  let v;
  const arr = new Uint8Array(16); // Parse ########-....-....-....-............

  arr[0] = (v = parseInt(uuid.slice(0, 8), 16)) >>> 24;
  arr[1] = v >>> 16 & 0xff;
  arr[2] = v >>> 8 & 0xff;
  arr[3] = v & 0xff; // Parse ........-####-....-....-............

  arr[4] = (v = parseInt(uuid.slice(9, 13), 16)) >>> 8;
  arr[5] = v & 0xff; // Parse ........-....-####-....-............

  arr[6] = (v = parseInt(uuid.slice(14, 18), 16)) >>> 8;
  arr[7] = v & 0xff; // Parse ........-....-....-####-............

  arr[8] = (v = parseInt(uuid.slice(19, 23), 16)) >>> 8;
  arr[9] = v & 0xff; // Parse ........-....-....-....-############
  // (Use "/" to avoid 32-bit truncation when bit-shifting high-order bytes)

  arr[10] = (v = parseInt(uuid.slice(24, 36), 16)) / 0x10000000000 & 0xff;
  arr[11] = v / 0x100000000 & 0xff;
  arr[12] = v >>> 24 & 0xff;
  arr[13] = v >>> 16 & 0xff;
  arr[14] = v >>> 8 & 0xff;
  arr[15] = v & 0xff;
  return arr;
}

var _default = parse;
exports.default = _default;
});

unwrapExports(parse_1);

var v35 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;
exports.URL = exports.DNS = void 0;

var _stringify = _interopRequireDefault(stringify_1);

var _parse = _interopRequireDefault(parse_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function stringToBytes(str) {
  str = unescape(encodeURIComponent(str)); // UTF8 escape

  const bytes = [];

  for (let i = 0; i < str.length; ++i) {
    bytes.push(str.charCodeAt(i));
  }

  return bytes;
}

const DNS = '6ba7b810-9dad-11d1-80b4-00c04fd430c8';
exports.DNS = DNS;
const URL = '6ba7b811-9dad-11d1-80b4-00c04fd430c8';
exports.URL = URL;

function _default(name, version, hashfunc) {
  function generateUUID(value, namespace, buf, offset) {
    if (typeof value === 'string') {
      value = stringToBytes(value);
    }

    if (typeof namespace === 'string') {
      namespace = (0, _parse.default)(namespace);
    }

    if (namespace.length !== 16) {
      throw TypeError('Namespace must be array-like (16 iterable integer values, 0-255)');
    } // Compute hash of namespace and value, Per 4.3
    // Future: Use spread syntax when supported on all platforms, e.g. `bytes =
    // hashfunc([...namespace, ... value])`


    let bytes = new Uint8Array(16 + value.length);
    bytes.set(namespace);
    bytes.set(value, namespace.length);
    bytes = hashfunc(bytes);
    bytes[6] = bytes[6] & 0x0f | version;
    bytes[8] = bytes[8] & 0x3f | 0x80;

    if (buf) {
      offset = offset || 0;

      for (let i = 0; i < 16; ++i) {
        buf[offset + i] = bytes[i];
      }

      return buf;
    }

    return (0, _stringify.default)(bytes);
  } // Function#name is not settable on some platforms (#270)


  try {
    generateUUID.name = name; // eslint-disable-next-line no-empty
  } catch (err) {} // For CommonJS default export support


  generateUUID.DNS = DNS;
  generateUUID.URL = URL;
  return generateUUID;
}
});

unwrapExports(v35);
v35.URL;
v35.DNS;

var md5_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _crypto = _interopRequireDefault(crypto__default);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function md5(bytes) {
  if (Array.isArray(bytes)) {
    bytes = Buffer.from(bytes);
  } else if (typeof bytes === 'string') {
    bytes = Buffer.from(bytes, 'utf8');
  }

  return _crypto.default.createHash('md5').update(bytes).digest();
}

var _default = md5;
exports.default = _default;
});

unwrapExports(md5_1);

var v3_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _v = _interopRequireDefault(v35);

var _md = _interopRequireDefault(md5_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const v3 = (0, _v.default)('v3', 0x30, _md.default);
var _default = v3;
exports.default = _default;
});

unwrapExports(v3_1);

var v4_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _rng = _interopRequireDefault(rng_1);

var _stringify = _interopRequireDefault(stringify_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function v4(options, buf, offset) {
  options = options || {};

  const rnds = options.random || (options.rng || _rng.default)(); // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`


  rnds[6] = rnds[6] & 0x0f | 0x40;
  rnds[8] = rnds[8] & 0x3f | 0x80; // Copy bytes to buffer, if provided

  if (buf) {
    offset = offset || 0;

    for (let i = 0; i < 16; ++i) {
      buf[offset + i] = rnds[i];
    }

    return buf;
  }

  return (0, _stringify.default)(rnds);
}

var _default = v4;
exports.default = _default;
});

unwrapExports(v4_1);

var sha1_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _crypto = _interopRequireDefault(crypto__default);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function sha1(bytes) {
  if (Array.isArray(bytes)) {
    bytes = Buffer.from(bytes);
  } else if (typeof bytes === 'string') {
    bytes = Buffer.from(bytes, 'utf8');
  }

  return _crypto.default.createHash('sha1').update(bytes).digest();
}

var _default = sha1;
exports.default = _default;
});

unwrapExports(sha1_1);

var v5_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _v = _interopRequireDefault(v35);

var _sha = _interopRequireDefault(sha1_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const v5 = (0, _v.default)('v5', 0x50, _sha.default);
var _default = v5;
exports.default = _default;
});

unwrapExports(v5_1);

var nil = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _default = '00000000-0000-0000-0000-000000000000';
exports.default = _default;
});

unwrapExports(nil);

var version_1 = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _validate = _interopRequireDefault(validate_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function version(uuid) {
  if (!(0, _validate.default)(uuid)) {
    throw TypeError('Invalid UUID');
  }

  return parseInt(uuid.substr(14, 1), 16);
}

var _default = version;
exports.default = _default;
});

unwrapExports(version_1);

var dist = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "v1", {
  enumerable: true,
  get: function () {
    return _v.default;
  }
});
Object.defineProperty(exports, "v3", {
  enumerable: true,
  get: function () {
    return _v2.default;
  }
});
Object.defineProperty(exports, "v4", {
  enumerable: true,
  get: function () {
    return _v3.default;
  }
});
Object.defineProperty(exports, "v5", {
  enumerable: true,
  get: function () {
    return _v4.default;
  }
});
Object.defineProperty(exports, "NIL", {
  enumerable: true,
  get: function () {
    return _nil.default;
  }
});
Object.defineProperty(exports, "version", {
  enumerable: true,
  get: function () {
    return _version.default;
  }
});
Object.defineProperty(exports, "validate", {
  enumerable: true,
  get: function () {
    return _validate.default;
  }
});
Object.defineProperty(exports, "stringify", {
  enumerable: true,
  get: function () {
    return _stringify.default;
  }
});
Object.defineProperty(exports, "parse", {
  enumerable: true,
  get: function () {
    return _parse.default;
  }
});

var _v = _interopRequireDefault(v1_1);

var _v2 = _interopRequireDefault(v3_1);

var _v3 = _interopRequireDefault(v4_1);

var _v4 = _interopRequireDefault(v5_1);

var _nil = _interopRequireDefault(nil);

var _version = _interopRequireDefault(version_1);

var _validate = _interopRequireDefault(validate_1);

var _stringify = _interopRequireDefault(stringify_1);

var _parse = _interopRequireDefault(parse_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
});

unwrapExports(dist);
var dist_1 = dist.v4;

function parseOrderRecursive(list) {
    let order = [];
    for (let i = 0; i < list.items.length; i++) {
        order.push({
            id: list.items[i].value.id,
            children: parseOrderRecursive(list.items[i].children),
        });
    }
    return order;
}
const useTree = ({ items = [], load = async () => { } }) => {
    const list = vue.reactive({
        items: [],
        isLoading: false,
        __changeHandlers: [],
        originalOrder: null,
        onOrderChange(handler) {
            this.__changeHandlers.push(handler);
        },
        load() {
            if (!load)
                throw new Error('Missing load function for tree.');
            this.isLoading = true;
            return load
                .apply(null, arguments)
                .then(response => {
                this.setItems(response.data.data);
                this.originalOrder = useOriginal(list.getOrder());
                this.isLoading = false;
                return response;
            })
                .catch(e => {
                this.isBusyLoading = false;
                throw e;
            });
        },
        push(item, children = []) {
            this.items.push({
                children: useTree({ items: children }),
                value: item,
            });
        },
        pop() {
            return this.items.pop();
        },
        setItems(list) {
            let items = [];
            for (let i = 0; i < list.length; i++) {
                items.push({
                    children: useTree({ items: list[i].children }),
                    uuid: dist_1(),
                    value: list[i].value,
                });
            }
            this.items = items;
        },
        getOrder() {
            return parseOrderRecursive(this);
        },
    });
    list.setItems(items);
    list.updateOnChange = items => {
        vue.watch(items, () => {
            list.setItems(typeof items === 'function' ? items() : items);
        }, { immediate: true, deep: true });
    };
    vue.watch(() => list, () => {
        // skip when list is loading or initial order has not yet been set
        if (list.isLoading || !list.originalOrder) {
            return;
        }
        const order = list.getOrder();
        if (list.originalOrder.matches(order))
            return;
        list.originalOrder.update(order);
        if (list.isLoading)
            return;
        for (let i = 0; i < list.__changeHandlers.length; i++) {
            list.__changeHandlers[i](order);
        }
    }, { immediate: true, deep: true });
    return list;
};

const useState = (value, { load }) => {
    return vue.reactive({
        value,
        isLoading: false,
        isLoaded: false,
        load() {
            this.isLoading = true;
            return load
                .apply(null, arguments)
                .then(response => {
                this.value = response.data.data;
                this.isLoading = false;
                this.isLoaded = true;
                return this.value;
            })
                .catch(e => {
                this.isLoading = false;
                throw e;
            });
        },
    });
};

const useSaveQueue = function () {
    const saveQueue = vue.reactive({
        jobs: {},
        isDirty: false,
        add(key, job, callback = () => null) {
            this.jobs[key] = { job, callback };
        },
        remove(key) {
            delete this.jobs[key];
        },
        async save() {
            let promises = [];
            for (let key in this.jobs) {
                let func = async () => {
                    await this.jobs[key].job();
                    this.jobs[key].callback();
                };
                promises.push(func());
            }
            this.jobs = {};
            await Promise.all(promises.map(p => p.catch(e => e)));
        },
    });
    vue.watch(saveQueue, () => {
        saveQueue.isDirty = Object.keys(saveQueue.jobs).length > 0;
    }, {
        // force eager callback execution
        immediate: true
    });
    return saveQueue;
};

function getValue(values, locale) {
    return (values.value)[locale.value] || null;
}
const translatable = (locale, values, onChange) => {
    const vals = vue.ref(values || {});
    const val = vue.ref(getValue(vals, locale));
    vue.watch(vals, (v) => onChange(v), { deep: true });
    vue.watch(locale, () => val.value = getValue(vals, locale), { deep: true });
    vue.watch(val, (v) => vals.value[locale.value] = v);
    return val;
};

const Input = function ({ modelModifiers = {} }, { attrs = {}, emit }) {
    let onInput = ({ target }) => {
        emit('update:modelValue', target.value);
    };
    if (modelModifiers.debounce) {
        onInput = ___namespace.debounce(onInput, 200);
    }
    return vue.h(`input`, {
        ...attrs,
        value: attrs.modelValue,
        onInput,
    });
};

const template$1 = `
    <Draggable
        v-if="value"
        v-model="value"
        v-bind="dragOptions"
        :group="group"
        itemKey="uuid"
        :component-data="{
            name: !drag ? 'flip-list' : null,
            wrap: true,
        }"
        @start="drag = true"
        @end="drag = false"
        handle=".handle"
    >
        <template #item="{ element }">
            <div>
                <component
                    :is="sections[getSectionKey(element.component)]"
                    :modelValue="element.value"
                    @update:modelValue="(e) => updateElement(element, e)"
                    @destroy="(e) => removeElement(element)"
                />
            </div>
        </template>
    </Draggable>
`;
const Sections = vue.defineComponent({
    template: template$1,
    components: { Draggable: Draggable__default },
    emits: ['update:modelValue'],
    props: {
        modelValue: {
            type: Array,
            required: true,
        },
        sections: {
            type: Object,
            required: true,
        },
        group: {
            type: String,
            default: 'sections',
        },
        dragOptions: {
            type: Object,
            default: () => ({
                animation: 200,
                ghostClass: 'ghost',
            }),
        },
    },
    setup(props, { emit }) {
        const drag = vue.ref(false);
        const updateElement = (element, value) => {
            element.value = value;
        };
        const getSectionKey = (section) => {
            const json = JSON.stringify(section);
            for (let key in props.sections) {
                if (JSON.stringify(props.sections[key]) == json) {
                    return key;
                }
            }
            return '';
        };
        const removeElement = element => {
            value.value = value.value.filter(section => {
                return section.uuid != element.uuid;
            });
        };
        const parseValue = (value) => {
            let b = [];
            for (let i in value) {
                b.push({
                    uuid: dist_1(),
                    key: value[i].type,
                    value: value[i].value,
                    component: props.sections[value[i].type],
                });
            }
            return b;
        };
        const defaultValue = (value, i) => {
            let defaultValue = value[i].component.props.modelValue.default();
            if (Object.keys(defaultValue).length < 1) {
                console.error('No default keys set in Section');
            }
            return defaultValue;
        };
        const transformValue = (value) => {
            let b = [];
            for (let i in value) {
                b.push({
                    type: getSectionKey(value[i].component),
                    value: value[i].value || defaultValue(value, i),
                });
            }
            return b;
        };
        const value = vue.ref(parseValue(props.modelValue));
        vue.watch(() => value, () => emit('update:modelValue', transformValue(value.value)), { deep: true });
        return { drag, updateElement, removeElement, getSectionKey, value };
    },
});

const template = `
    <Draggable
        v-model="drawers"
        :group="{
            name: group,
            pull: 'clone',
            put: false,
        }"
        :sort="false"
        :clone="cloneSection"
        item-key="id"
    >
        <template #item="{ element }">
            <div>
                <component :is="element" />
            </div>
        </template>
    </Draggable>
`;
const Cabinet = vue.defineComponent({
    template,
    components: { Draggable: Draggable__default },
    props: { group: { type: String, default: 'sections' } },
    setup({}, { slots }) {
        const drawers = vue.ref();
        vue.onMounted(() => {
            if (slots.default) {
                drawers.value = slots.default();
            }
        });
        const cloneSection = (el) => ({
            uuid: dist_1(),
            component: el.props.draws,
        });
        return { cloneSection, drawers };
    },
});

exports.Cabinet = Cabinet;
exports.Input = Input;
exports.Sections = Sections;
exports.translatable = translatable;
exports.useForm = useForm;
exports.useIndex = useIndex;
exports.useOriginal = useOriginal;
exports.useSaveQueue = useSaveQueue;
exports.useState = useState;
exports.useTree = useTree;
